/**
  *  \file client/widgets/allylevelgrid.cc
  *  \brief Alliance Level Grid
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <stddef.h>
#include "client/widgets/allylevelgrid.h"
#include "game/alliance.h"
#include "gfx/gfxcontext.h"
#include "gfx/palette.h"
#include "resmgr/resmgr.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "util/nls.h"
#include "util/strtab.h"

/*
 * Layout:
 *    ---- Our offer
 *   |  +- Their offer
 *  [x][x] Level
 *  [x][x] Level
 *  [x][x] Level
 *  [x][x] Level
 *
 *  Checkboxes are 16x16, plus a border makes 20x20; UICheckbox reserves 24x24.
 *  Labels are in FONT_SMALL, so add 2 lines of that.
 */

static const char*const LABEL_TEXT[] = {
    N_("Our offer"),
    N_("Their offer"),
};
static const size_t NUM_LABELS = countof(LABEL_TEXT);

static const int GRID_SIZE = 24;

static void
drawCheckbox(GfxContext& ctx, int x, int y, GAllianceOffer::OfferType type, bool selected)
{
    Ptr<GfxPixmap> pix;
    switch (type) {
     case GAllianceOffer::Unknown:
     case GAllianceOffer::No:
        pix = requestPixmap("cb1");
        break;

     case GAllianceOffer::Yes:
        pix = requestPixmap("cb2");
        break;

     case GAllianceOffer::Conditional:
        pix = requestPixmap("cbc");
        break;
    }

    if (pix) {
        pix->blit(ctx.getCanvas(), x + (GRID_SIZE-16)/2, y + (GRID_SIZE-16)/2);
    }

    ctx.setColor(selected ? UIColor::tc_Static : UIColor::tc_Background);
    drawRectangle(ctx, GfxRect(x, y, GRID_SIZE, GRID_SIZE));
}

/** Constructor.
    \param id Widget Id
    \param editAllies Alliance object */
WAllyLevelGrid::WAllyLevelGrid(int id, GAlliances& editAllies)
    : UIWidget(id),
      mouseDown(false),
      player(1),
      position(0),
      editAllies(editAllies)
{ }

void
WAllyLevelGrid::getLayoutInfo(LayoutInfo& info)
{
    const GAllianceLevels& levels = editAllies.getLevels();

    GfxPoint checkboxSize = GfxPoint(GRID_SIZE, GRID_SIZE).scaledBy(2, levels.size());

    int labelWidth = getTextWidth(*fonts[FONT_SMALL], StaticTranslatedStringTable(LABEL_TEXT, NUM_LABELS));
    for (size_t i = 0, n = levels.size(); i < n; ++i) {
        labelWidth = std::max(labelWidth, fonts[FONT_TITLE]->getTextWidth(levels[i].getName()));
    }

    info.pref_size = info.min_size = GfxPoint(labelWidth + checkboxSize.x, NUM_LABELS*font_heights[FONT_SMALL] + checkboxSize.y);
    info.flags = 0;
}

void
WAllyLevelGrid::drawContent(GfxCanvas& can)
{
    // Make a context
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());

    // Header
    ctx.useFont(*fonts[FONT_SMALL]);
    const int headerHeight = font_heights[FONT_SMALL];
    for (size_t i = 0; i < NUM_LABELS; ++i) {
        ctx.setColor(UIColor::tc_Faded);
        drawHLine(ctx,
                  extent.x + GRID_SIZE/2 + i*GRID_SIZE,
                  extent.y + headerHeight/2 + i*headerHeight,
                  extent.x + GRID_SIZE*NUM_LABELS - 2);
        drawVLine(ctx,
                  extent.x + GRID_SIZE/2 + i*GRID_SIZE,
                  extent.y + headerHeight/2 + i*headerHeight,
                  extent.y + headerHeight*NUM_LABELS - 2);
        ctx.setColor(UIColor::tc_Static);
        outText(ctx, extent.x + GRID_SIZE*NUM_LABELS, extent.y + headerHeight*i, _(LABEL_TEXT[i]));
    }

    // Checkboxes and Labels
    ctx.useFont(*fonts[FONT_TITLE]);
    ctx.setTextAlign(0, 1);
    const GAllianceLevels& levels = editAllies.getLevels();
    bool focus = getFocusState() != NoFocus;
    for (size_t i = 0, n = levels.size(); i < n; ++i) {
        int y = extent.y + headerHeight*NUM_LABELS + i*GRID_SIZE;
        if (const GAllianceOffer* pOffer = editAllies.getOffer(i)) {
            drawCheckbox(ctx, extent.x,             y, pOffer->newOffer[player],   focus && position == int(i));
            drawCheckbox(ctx, extent.x + GRID_SIZE, y, pOffer->theirOffer[player], false);
        }
        ctx.setColor(UIColor::tc_Static);
        outText(ctx, extent.x + GRID_SIZE*NUM_LABELS, y + GRID_SIZE/2, levels[i].getName());
    }

    can.update(getExtent());
}

bool
WAllyLevelGrid::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Key && hasState(st_Focused)) {
        switch (event.key.code) {
         case SDLK_UP:
            setSelectedWidget();
            setPosition(position - 1);
            return true;

         case SDLK_DOWN:
            setSelectedWidget();
            setPosition(position + 1);
            return true;

         case SDLK_HOME:
         case SDLK_PAGEUP:
            setSelectedWidget();
            setPosition(0);
            return true;

         case SDLK_END:
         case SDLK_PAGEDOWN:
            setSelectedWidget();
            setPosition(editAllies.getLevels().size());
            return true;

         case ' ':
            setSelectedWidget();
            toggleCurrent();
            return true;
        }
    }
    if (event.type == ev_Mouse) {
        int y = getExtent().y + NUM_LABELS*font_heights[FONT_SMALL];
        int x = getExtent().x;
        int n = editAllies.getLevels().size();
        if (event.mouse.x >= x && event.mouse.y >= y && event.mouse.x < x + GRID_SIZE && event.mouse.y < y + n*GRID_SIZE) {
            // Mouse is inside left column of checkboxes
            setSelectedWidget();
            int line = (event.mouse.y - y) / GRID_SIZE;
            if (event.mouse.b) {
                setFocusedWidget();
                mouseDown = true;
                setPosition(line);
            } else {
                if (mouseDown) {
                    toggleCurrent();
                }
                mouseDown = false;
            }
            return true;
        } else {
            mouseDown = false;
            // Do not eat this event.
        }
    }
    return UIWidget::handleEvent(event, second_pass);
}

void
WAllyLevelGrid::onStateChange(int state, bool enable)
{
    UIWidget::onStateChange(state, enable);
    if (state & st_Focused) {
        // Must redraw if focus changes
        drawWidget();
    }
    if (state & st_Selected) {
        // Forget mouse is down if anyone else claims it
        mouseDown = false;
    }
}

/** Set current player.
    This defines the player whose alliances are shown in the checkbox grid.
    \param player Player number */
void
WAllyLevelGrid::setPlayer(int player)
{
    if (player != this->player) {
        this->player = player;
        drawWidget();
    }
}

/** Set position in checkbox grid.
    The position is clamped into range.
    \param newPos new position, 0-based */
void
WAllyLevelGrid::setPosition(int newPos)
{
    if (newPos < 0) {
        newPos = 0;
    }
    if (newPos >= int(editAllies.getLevels().size())) {
        newPos = editAllies.getLevels().size() - 1;
    }
    if (newPos != position) {
        position = newPos;
        drawWidget();
    }
}

/** Process a click onto the current checkbox. */
void
WAllyLevelGrid::toggleCurrent()
{
    const GAllianceLevel* level = editAllies.getLevel(position);
    const GAllianceOffer* offer = editAllies.getOffer(position);
    if (level && offer) {
        // Cycling order is (Unknown/No) -> (Yes) -> (Conditional)
        switch (offer->newOffer[player]) {
         case GAllianceOffer::No:
         case GAllianceOffer::Unknown:
            editAllies.set(position, player, GAllianceOffer::Yes);
            if (level->hasFlag(GAllianceLevel::NeedsOffer)) {
                editAllies.setAll(player, GAllianceLevel::IsOffer, true);
            }
            break;

         case GAllianceOffer::Yes:
            if (level->hasFlag(GAllianceLevel::AllowConditional)) {
                editAllies.set(position, player, GAllianceOffer::Conditional);
            } else {
                editAllies.set(position, player, GAllianceOffer::No);
            }
            break;

         case GAllianceOffer::Conditional:
            editAllies.set(position, player, GAllianceOffer::No);
            break;
        }
    }
}

