/**
  *  \file flak/vis-ui.cc
  *  \brief User Interface to FLAK Player
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <vector>
#include <algorithm>
#include "cpluslib/format.h"
#include "cpluslib/sigslot.h"
#include "flak/vis.h"
#include "game/spec.h"
#include "gfx/clip.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "flak/vis-ui.h"
#include "game/racename.h"
#include "util/recode.h"
#include "util/misc.h"

enum WhichVis {
    vis_Flat,
    vis_Iso,
    vis_Last = vis_Iso,
    vis_First = vis_Flat
};

static const char*const VISUALIZER_NAMES[] = {
    N_("Flat View"),
    N_("Isometric 3D View")
};

// static const int TAB_SIZE  = 160;

static WhichVis visualizer = vis_Iso;

static const int FLAK_SHIP_DPY_WIDTH = 280; // FIXME

int
FlakShipList::Item::getHeight() const
{
    if (!visible)
        return false;
    return font_heights[FONT];
}

/*************************** FlakSimpleDivider ***************************/

void
FlakSimpleDivider::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    const GfxRect& e = getExtent();
    drawBackground(ctx, GfxRect(e.x+2, e.y, e.w-2, e.h));
    drawSolidBar(can, GfxRect(e.x, e.y, 2, e.h), standard_colors[COLOR_BLACK]);
    can.update(getExtent());
}

bool
FlakSimpleDivider::handleEvent(const UIEvent& /*e*/, bool /*second_pass*/)
{
    return false;
}

void
FlakSimpleDivider::getLayoutInfo(LayoutInfo& li)
{
    li.min_size = li.pref_size = GfxPoint(4, 4);
    li.flags = li_FlexibleV;
}

/****************************** FlakShipList *****************************/

/*! \class FlakShipList
    \brief List of ships in a fight */

FlakShipList::FlakShipList(FlakBattle& battle)
    : battle(battle), current(0), top(0), num_dead_units(0)
{
    for (FlakBattle::fleet_t i = 0; i < battle.getNumFleets(); ++i) {
        const FlakFleet& fl = battle.getFleetByNumber(i);
        content.push_back(Item(i, Item::fFleet, true));
        for (FlakBattle::ship_t s = 0; s < fl.data.num_ships; ++s)
            content.push_back(Item(s + fl.data.first_ship, Item::fShip, false));
    }
}

void
FlakShipList::onUpdate()
{
    unsigned count = 0;
    for (FlakBattle::ship_t i = 0, max = battle.getNumShips(); i < max; ++i)
        if (!battle.getShipByNumber(i).isAlive())
            ++count;

    if (count != num_dead_units) {
        num_dead_units = count;
        drawWidget(false);
    }
}

void
FlakShipList::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.setSolidBackground();
    ctx.useFont(*fonts[FONT]);
    GfxRect r = getExtent();

    unsigned item = top;
    int bot = r.y + r.h;
    while (r.y < bot) {
        r.h = bot - r.y;
        if (item >= content.size()) {
            // end of list reached
            drawBackground(ctx, r);
            break;
        } else {
            // list item
            if (content[item].visible) {
                // visible list item
                string_t the_text;
                bool alive = false;
                switch (content[item].what) {
                 case Item::fFleet: {
                    const FlakFleet& f = battle.getFleetByNumber(content[item].index);
                    the_text = format(_(" %d %s unit%0$!d%!1{s%}"), f.data.num_ships, player_racenames.getAdjName(f.data.player));
                    alive = f.isAlive();
                    break;
                 }
                 case Item::fShip: {
                    const FlakShip& sh = battle.getShipByNumber(content[item].index);
                    the_text = format("   %s (#%d)", convertGameToUtf8(getBasicString(sh.data.name)), sh.data.id);
                    alive = sh.isAlive();
                    break;
                 }
                 default:
                    FAIL();
                }

                // display it
                if (item == current) {
                    ctx.setTransparentBackground();
                    drawSolidBar(ctx, GfxRect(r.x, r.y, r.w, font_heights[FONT]), UIColor::tc_Static);
                    ctx.setColor(alive ? UIColor::tc_White : UIColor::tc_Faded);
                } else {
                    ctx.setSolidBackground();
                    ctx.setColor(alive ? UIColor::tc_Static : UIColor::tc_Faded);
                }
                outTextF(ctx, r.x, r.y, r.w, the_text);
                r.y += content[item].getHeight();
            } else {
                // invisible, do nothing
            }
            ++item;
        }
    }
    can.update(getExtent());
}

void
FlakShipList::getLayoutInfo(LayoutInfo& li)
{
    li.min_size = li.pref_size = font_sizes[FONT].scaledBy(12, 20);
    li.flags = li_FlexibleV;
}

FlakBattle::ship_t
FlakShipList::getSelectedShip() const
{
    if (content[current].what == Item::fShip)
        return content[current].index;
    else
        return FlakBattle::no_ship;
}

void
FlakShipList::toggleFleetVisibility()
{
    unsigned i = current + 1;
    if (i >= content.size())
        return;
    bool new_value = !content[i].visible;
    while (i < content.size() && content[i].what != Item::fFleet)
        content[i++].visible = new_value;
    adjustTop(); // just in case
    drawWidget(false);
}

bool
FlakShipList::handleEvent(const UIEvent& e, bool /*second_pass*/)
{
    if (e.type == ev_Mouse && e.mouse.b && getExtent().contains(e.mouse.x, e.mouse.y)) {
        setSelectedWidget();
        int diff = e.mouse.y - getExtent().y;
        unsigned index = top;
        while (index < content.size()) {
            int here = content[index].getHeight();
            if (here > diff)
                break;
            diff -= here;
            ++index;
        }
        if (index < content.size() && index != current) {
            current = index;
            adjustTop();
            drawWidget(false);
            sig_moved.raise();
        }
        return true;
    } else if (e.type == ev_Key) {
        switch (e.key.code) {
         case ' ':
            setSelectedWidget();
            if (content[current].what == Item::fFleet)
                toggleFleetVisibility();
            else if (content[current].what == Item::fShip)
                sig_ship_clicked.raise(content[current].index);
            return true;
         case SDLK_UP:
            scrollBy(-1);
            return true;
         case SDLK_DOWN:
            scrollBy(+1);
            return true;
        }
    }
    return false;   
}

void
FlakShipList::scrollBy(int diff)
{
    int np = current;
    do {
        np += diff;
        if (np < 0 || np >= int(content.size()))
            return;
    } while (!content[np].visible);
    current = np;
    adjustTop();
    drawWidget(false);
    sig_moved.raise();    
}

void
FlakShipList::setFleetStatus(FlakBattle::fleet_t fleet, bool status)
{
    for (unsigned i = 0; i < content.size(); ++i) {
        if (content[i].what == Item::fFleet && content[i].index == fleet) {
            setFleetStatusByPosition(i, status);
            break;
        }
    }
}

void
FlakShipList::setFleetStatusByPosition(unsigned index, bool status)
{
    ASSERT(content[index].what == Item::fFleet);
    bool changed = false;
    for (unsigned i = index+1; i < content.size() && content[i].what != Item::fFleet; ++i) {
        if (content[i].visible != status) {
            content[i].visible = status;
            changed = true;
            if (!status && i == current)
                current = index;
        }
    }
    if (changed) {
        adjustTop();
        drawWidget(false);
    }
}

/** Show ship and place cursor on it. */
void
FlakShipList::showShip(FlakBattle::ship_t ship)
{
    // first, find the entry which contains the ship. While doing so,
    // keep track of the fleet it is in.
    int fleet_index = -1, ship_index = -1;
    for (unsigned i = 0; i < content.size(); ++i) {
        if (content[i].what == Item::fFleet) {
            fleet_index = i;
        } else if (content[i].what == Item::fShip && content[i].index == ship) {
            ship_index = i;
            break;
        }
    }

    // found it?
    if (ship_index >= 0 && fleet_index >= 0) {
        if (!content[ship_index].visible) {
            // show it
            for (unsigned i = fleet_index+1; i < content.size() && content[i].what == Item::fShip; ++i)
                content[i].visible = true;
        }
        // move cursor to it
        if (current != ship_index) {
            current = ship_index;
            adjustTop();
            drawWidget(false);
            sig_moved.raise();
        }
    }
}

/** Adjust top of window such that current selection is visible. */
void
FlakShipList::adjustTop()
{
    if (current < top) {
        top = current;
        drawWidget(false);
    } else if (current > top) {
        // compute distance of current item to bottom
        int dist = 0;
        int max_height = getExtent().h;
        for (unsigned index = top; index <= current; ++index)
            dist += content[index].getHeight();
        if (dist > max_height) {
            while (current > top && dist > max_height) {
                dist -= content[top].getHeight();
                ++top;
            }
            drawWidget(false);
        }
    }
}

static void
drawStandardFrame(GfxCanvas& can, GfxRect r)
{
    // FIXME: almost dupe from chartusr.cc
    const int icolor = standard_colors[COLOR_SHIELD+3];
    const int fcolor = standard_colors[COLOR_SHIELD+5];
    const int z = standard_colors[COLOR_BLACK];

    int alpha = ui_screen_bits <= 8 ? GFX_ALPHA_OPAQUE : (3*GFX_ALPHA_OPAQUE + GFX_ALPHA_TRANSPARENT)/4;

    can.drawBar(GfxRect(r.x+1,     r.y,       r.w-2, 2),     fcolor, z, SOLID_FILL, alpha);
    can.drawBar(GfxRect(r.x+1,     r.y+r.h-2, r.w-2, 2),     fcolor, z, SOLID_FILL, alpha);
    can.drawBar(GfxRect(r.x,       r.y+2,     2,     r.h-4), fcolor, z, SOLID_FILL, alpha);
    can.drawBar(GfxRect(r.x+r.w-2, r.y+2,     2,     r.h-4), fcolor, z, SOLID_FILL, alpha);
    can.drawPixel(GfxPoint(r.x,       r.y+1),     fcolor, alpha);
    can.drawPixel(GfxPoint(r.x+r.w-1, r.y+1),     fcolor, alpha);
    can.drawPixel(GfxPoint(r.x,       r.y+r.h-2), fcolor, alpha);
    can.drawPixel(GfxPoint(r.x+r.w-1, r.y+r.h-2), fcolor, alpha);

    r.grow(-2, -2);
    if (ui_screen_bits <= 8)
        can.drawBar(r, icolor, z, GRAY50_FILL, GFX_ALPHA_OPAQUE);
    else
        can.drawBar(r, icolor, z, SOLID_FILL, 128);
}

FlakTimeDisplay::FlakTimeDisplay(const GfxRect& player_size, FlakBattle& b, uint32 max_time)
    : GfxSprite3D(FlakFancyVisualizer::layer_Text),
      battle(b), max_time(max_time), digits(0)
{
    // FIXME: we should actually use this computed value...
    uint32 mins = max_time / 60;
    while (mins)
        mins /= 10, ++digits;

    GfxRect new_pos(0, 0,
                    fonts[FONT_FIXED]->getTextWidth("000:00 (999%)") + 8,
                    font_heights[FONT_FIXED] + 8);
    new_pos.y = player_size.getBottomY() - new_pos.h;
    new_pos.x = player_size.getRightX() - new_pos.w;
    setPos(new_pos, 0);
}

void
FlakTimeDisplay::tick()
{
    update();
}

void
FlakTimeDisplay::draw(GfxCanvas& can)
{
    uint32 time = battle.getTime();

    drawStandardFrame(can, getPos());

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.useFont(*fonts[FONT_FIXED]);
    ctx.setColor(COLOR_WHITE);
    if (max_time > 0) {
        outTextF(ctx, getPos().x + 4, getPos().y + 4, getPos().w - 8,
                 format("%d:%02d (%d%%)", time / 60, time % 60, time * 100 / max_time));
    } else {
        outTextF(ctx, getPos().x + 4, getPos().y + 4, getPos().w - 8,
                 format("%d:%02d", time / 60, time % 60, time * 100));
    }
}

/************************** FlakDistanceDisplay **************************/

FlakDistanceDisplay::FlakDistanceDisplay(const GfxRect& player_size, FlakPlayWidget& widget)
    : GfxSprite3D(FlakFancyVisualizer::layer_Text),
      widget(widget)
{
    GfxRect new_pos (player_size.x,
                     0,
                     fonts[FONT_FIXED]->getTextWidth(_("Distance: 000000 m")) + 8,
                     font_heights[FONT_FIXED] + 8);
    new_pos.y = player_size.getBottomY() - new_pos.h;
    setPos(new_pos, 0);
}

void
FlakDistanceDisplay::tick()
{
    update();
}

void
FlakDistanceDisplay::draw(GfxCanvas& can)
{
    FlakBattle::ship_t a = widget.getShipTrack(0), b = widget.getShipTrack(1);

    if (a != FlakBattle::no_ship && b != FlakBattle::no_ship) {
        FlakBattle& battle = widget.getBattle();
        FlakShip& ash = battle.getShipByNumber(a);
        FlakShip& bsh = battle.getShipByNumber(b);
        if (ash.isAlive() && bsh.isAlive()) {
            double dist = ash.getPos().distanceTo(bsh.getPos());
            drawStandardFrame(can, getPos());
            GfxContext ctx;
            ctx.useCanvas(can);
            ctx.useColorScheme(GfxStandardColorScheme::instance);
            ctx.useFont(*fonts[FONT_FIXED]);
            ctx.setColor(COLOR_WHITE);
            outTextF(ctx, getPos().x + 4, getPos().y + 4, getPos().w - 8,
                     format(_("Distance: %6d m"), long(dist)));
        }
    }
}

/**************************** FlakShipDisplay ****************************/

void
FlakShipDisplay::tick()
{
    update();
}

void
FlakShipDisplay::draw(GfxCanvas& can)
{
    FlakShip& sh = battle.getShipByNumber(ship);
    GfxRect rect = getPos();
    drawStandardFrame(can, rect);
    rect.grow(-4, -4);

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(COLOR_YELLOW);
    outTextF(ctx, rect.x, rect.y, rect.w, convertGameToUtf8(getBasicString(sh.data.name)));
    rect.y += font_heights[FONT_NORMAL] + 2;
    ctx.setColor(COLOR_WHITE);
    if (sh.isPlanet()) {
        outTextF(ctx, rect.x, rect.y, rect.w,
                 format(_("#%d, a %s planet"), sh.data.id, player_racenames.getAdjName(sh.data.player)));
        rect.y += font_heights[FONT_NORMAL];
        if (!sh.isAlive())
            outTextF(ctx, rect.x, rect.y, rect.w, _("dead"));
        else
            outTextF(ctx, rect.x, rect.y, rect.w,
                     format(_("%d%% shield, %d%% damage"),
                            int(sh.status.shield),
                            int(sh.status.damage),
                            int(sh.status.crew)));
        rect.y += font_heights[FONT_NORMAL];
    } else {
        outTextF(ctx, rect.x, rect.y, rect.w,
                 format(_("#%d, a %s %s"), sh.data.id, player_racenames.getAdjName(sh.data.player), ::getHull(sh.data.hull).getName()));
        rect.y += font_heights[FONT_NORMAL];
        if (!sh.isAlive())
            outTextF(ctx, rect.x, rect.y, rect.w, _("dead"));
        else
            outTextF(ctx, rect.x, rect.y, rect.w,
                     format(_("%d%% shield, %d%% damage, %d crew"),
                            int(sh.status.shield),
                            int(sh.status.damage),
                            numToString(int(sh.status.crew))));
        rect.y += font_heights[FONT_NORMAL];
    }
    if (sh.data.torp_lcount > 0) {
        if (sh.data.bay_count > 0) {
            outTextF(ctx, rect.x, rect.y, rect.w,
                     format(_("%d fighter%!1{s%}, %d torpedo%!1{es%}"),
                            numToString(sh.status.fighter_count),
                            numToString(sh.status.torp_count)));
        } else {
            outTextF(ctx, rect.x, rect.y, rect.w,
                     format(_("%d torpedo%!1{es%}"),
                            numToString(sh.status.torp_count)));
        }
        rect.y += font_heights[FONT_NORMAL];
    } else if (sh.data.bay_count > 0) {
        outTextF(ctx, rect.x, rect.y, rect.w,
                 format(_("%d fighter%!1{s%}"),
                        numToString(sh.status.fighter_count)));
        rect.y += font_heights[FONT_NORMAL];
    }
}

void
FlakShipDisplay::adjustSize()
{
    const GfxRect& r = getPos();
    int height = 2*4 + 2 + 3*font_heights[FONT_NORMAL];
    FlakShip& sh = battle.getShipByNumber(ship);
    if (sh.data.torp_lcount > 0 || sh.data.bay_count > 0)
        height += font_heights[FONT_NORMAL];
    setPos(GfxRect(r.x, r.y + r.h - height, width, height), getPosZ());
}

void
FlakShipDisplay::setShipNumber(FlakBattle::ship_t ship)
{
    if (ship != this->ship) {
        this->ship = ship;
        update();
        adjustSize();
    }
}

/************************* FlakPlayControlWidget *************************/

FlakPlayControlWidget::FlakPlayControlWidget()
    : UIGroup(UINullLayout::instance /* will later be overridden by grid */),
      grid(5, 0, 0),
      rewind_start(0, "\xEE\x85\x84""\xE2\x97\x80", ss_Alt + SDLK_LEFT),
      rewind(0, "\xE2\x97\x80""\xE2\x97\x80", ss_Ctrl + SDLK_LEFT),
      play(0, "\xE2\x96\xB6", SDLK_RIGHT),
      forward(0, "\xE2\x96\xB6""\xE2\x96\xB6", ss_Ctrl + SDLK_RIGHT),
      forward_end(0, "\xE2\x96\xB6""\xEE\x85\x84", ss_Alt + SDLK_RIGHT)
{
    grid.setForcedCellSize(24, 24);
    setLayout(grid);
    add(rewind_start);
    add(rewind);
    add(play);
    add(forward);
    add(forward_end);
}

FlakPlayControlWidget::~FlakPlayControlWidget()
{ }

/************************* FlakMainControlWidget *************************/

FlakMainControlWidget::FlakMainControlWidget()
    : UIGroup(UIHBoxLayout::instance0),
      exit_button(0, "ESC", SDLK_ESCAPE)
{
    exit_button.addStop();
    add(spacer);
    add(exit_button);
}

FlakMainControlWidget::~FlakMainControlWidget()
{ }

void
FlakMainControlWidget::drawContent(GfxCanvas& can)
{
    drawBackground(GfxContext().useCanvas(can).useColorScheme(getSkin()), getExtent());
    can.update(getExtent());
}

/***************************** FlakPlayWidget ****************************/

FlakPlayWidget::FlakPlayWidget(FlakVcrEntry& entry, FlakShipList& list)
    : UIWidget(), entry(entry), vis(0),
      playing(false), mouse_down(false),
      ship_list(list)
{
    createVisualizer(true);
}

FlakPlayWidget::~FlakPlayWidget()
{
    delete vis;
}

void
FlakPlayWidget::onResize(GfxRect extent, bool by_layout)
{
    UIWidget::onResize(extent, by_layout);
    createVisualizer(false);
}

void
FlakPlayWidget::createVisualizer(bool rewind)
{
    GfxRect ps = getExtent();
    if (ps.w < 50)
        ps.w = 50;
    if (ps.h < 50)
        ps.h = 50;

    FlakFancyVisualizer* old_vis = vis;
    getBattle().setVisualizer(0);
    switch (visualizer) {
     case vis_Iso:
        vis = new FlakIsoVisualizer(getBattle(), ps);
        break;
     case vis_Flat:
        vis = new FlakFlatVisualizer(getBattle(), ps);
        break;
     default:
        FAIL();
    }

    if (old_vis != 0) {
        vis->swapStatusTokens(*old_vis);
        delete old_vis;
    }

    vis->init(rewind);
    vis->getManager().addSprite(new FlakTimeDisplay(ps, getBattle(), getTotalTime()));
    vis->getManager().addSprite(new FlakDistanceDisplay(ps, *this));
    if (ship_displays[0])
        vis->getManager().addSprite(ship_displays[0]);
    if (ship_displays[1])
        vis->getManager().addSprite(ship_displays[1]);
}

void
FlakPlayWidget::updateSprites()
{
    ObjectHolder h;
    vis->getManager().doUpdates(getParent()->getCanvasFor(*this, h));
}

void
FlakPlayWidget::setPlayStatus(bool playing)
{
    if (playing != this->playing) {
        this->playing = playing;
        sig_play_status.raise();
    }
}

void
FlakPlayWidget::setShipTrack(int side, FlakBattle::ship_t ship)
{
    if (!ship_displays[side]) {
        const int width = std::min(int(FLAK_SHIP_DPY_WIDTH), int((getExtent().w - 2) / 2));
        const int x = (side
                       ? getExtent().getCenter().x + 1
                       : getExtent().getCenter().x - width - 1);
        const int bot = getExtent().getBottomY() - 9 - font_heights[FONT_NORMAL]; // FIXME
        ship_displays[side] = new FlakShipDisplay(GfxPoint(x, bot), width, getBattle(), ship);
        vis->getManager().addSprite(ship_displays[side]);
    } else {
        ship_displays[side]->setShipNumber(ship);
    }
    updateSprites();
}

bool
FlakPlayWidget::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Tick) {
        if (!second_pass && playing && !vis->isTerminated()) {
            ObjectHolder h;
            vis->doTick(getParent()->getCanvasFor(*this, h));
            if (vis->isTerminated())
                setPlayStatus(false);
            sig_update.raise();
        }
    } else if (e.type == ev_Key) {
        FlakBattle::ship_t ship_id;
        switch (e.key.code) {
         case SDLK_TAB:
            setSelectedWidget();
            if (visualizer >= vis_Last)
                visualizer = vis_First;
            else
                visualizer = WhichVis(visualizer+1);
            createVisualizer(false);
            drawWidget();
            sig_update.raise();
            return true;
         case '1':
            setSelectedWidget();
            if ((ship_id = ship_list.getSelectedShip()) != FlakBattle::no_ship)
                setShipTrack(0, ship_id);
            return true;
         case '2':
            setSelectedWidget();
            if ((ship_id = ship_list.getSelectedShip()) != FlakBattle::no_ship)
                setShipTrack(1, ship_id);
            return true;
         case 'b':
            setTime(vis->getBattle().getTime() - 1);
            return true;
         case 'f':
            setTime(vis->getBattle().getTime() + 1);
            return true;
        }
    } else if (e.type == ev_Mouse && getExtent().contains(e.mouse.x, e.mouse.y)) {
        setSelectedWidget();
        if (e.mouse.b && !mouse_down) {
            FlakBattle::ship_t sh = vis->getShipAtPosition(GfxPoint(e.mouse.x, e.mouse.y));
            if (sh != FlakBattle::no_ship)
                ship_list.sig_ship_clicked.raise(sh);
            mouse_down = true;
        } else if (e.mouse.b == 0) {
            mouse_down = false;
        }
        return true;
    }

    switch (vis->handleEvent(e)) {
     case FlakFancyVisualizer::eRedrawMe:
        setSelectedWidget();
        drawWidget(true);
        return true;
     case FlakFancyVisualizer::eDontRedrawMe:
        setSelectedWidget();
        return true;
     case FlakFancyVisualizer::eNotHandled:
        return UIWidget::handleEvent(e, second_pass);
    }
    return false; // not reached
}

void
FlakPlayWidget::setTime(int32 time)
{
    if (time != vis->getBattle().getTime()) {
        setSelectedWidget();
        vis->setTime(time);
        updateSprites();
        setPlayStatus(false);
        sig_update.raise();
    }
}

void
FlakPlayWidget::onRewindToStart()
{
    setTime(0);
}

void
FlakPlayWidget::onRewind()
{
    setTime(vis->getBattle().getTime() - 20);
}

void
FlakPlayWidget::onPlay()
{
    setPlayStatus(!playing);
}

void
FlakPlayWidget::onForward()
{
    setTime(vis->getBattle().getTime() + 20);
}

void
FlakPlayWidget::onForwardToEnd()
{
    setTime(vis->EndOfFight);
}

void
FlakPlayWidget::onShipClicked(FlakBattle::ship_t ship)
{
    vis->setShipStatus(ship, !vis->isShipBoxed(ship));
    if (vis->isShipBoxed(ship)) {
        if (!ship_displays[0] && !ship_displays[1] && vis->getShipSprite(ship)->getPos().x >= getExtent().x + getExtent().w/2)
            setShipTrack(1, ship);
        else if (!ship_displays[0])
            setShipTrack(0, ship);
        else if (!ship_displays[1])
            setShipTrack(1, ship);
    }
    ship_list.showShip(ship);
    updateSprites();
}

void
FlakPlayWidget::drawContent(GfxCanvas& can)
{
    vis->draw(can);
    can.update(getExtent());
}

void
FlakPlayWidget::getLayoutInfo(LayoutInfo& li)
{
    li.min_size  = GfxPoint(100, 100);
    li.pref_size = GfxPoint(300, 300);
    li.flags = li_Flexible;
}

/** Play specified battle. */
void
playFlakBattle(FlakVcrEntry& battle)
{
    ObjectHolder h;
    UIGroup         group(UIHBoxLayout::instance0);
    UIGroup         vgroup(UIVBoxLayout::instance0);
    FlakShipList           list(battle.getBattle());
    FlakPlayWidget         wid(battle, list);
    FlakPlayControlWidget  control;
    group.setSkin(UISkin::getDefaultSkin());
    group.setExtent(ui_root->getExtent());
    group.add(wid);
    group.add(h.add(new FlakSimpleDivider()));
    group.add(vgroup);
    vgroup.add(list);
    vgroup.add(h.add(new FlakMainControlWidget()));
    vgroup.add(control);
    control.rewind_start.sig_fired.add(&wid, &FlakPlayWidget::onRewindToStart);
    control.rewind.sig_fired.add(&wid, &FlakPlayWidget::onRewind);
    control.play.sig_fired.add(&wid, &FlakPlayWidget::onPlay);
    control.forward.sig_fired.add(&wid, &FlakPlayWidget::onForward);
    control.forward_end.sig_fired.add(&wid, &FlakPlayWidget::onForwardToEnd);
    wid.sig_update.add(&list, &FlakShipList::onUpdate);
    list.sig_ship_clicked.add(&wid, &FlakPlayWidget::onShipClicked);
    group.doLayout();
    ui_root->add(group);
    ui_root->run();
}
