/**
  *  \file game/actions/planetsupplies.cc
  *  \brief Planetary Action: Supply Sale
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/actions/planetsupplies.h"
#include "util/nls.h"
#include "game/actions/preconditions.h"

/*********************** GPlanetSellSuppliesAction ***********************/

/** Constructor.
    \param the_planet planet
    \pre we are playing the planet */
GPlanetSellSuppliesAction::GPlanetSellSuppliesAction(GPlanet& the_planet)
    : the_planet(the_planet),
      transfer(the_planet),
      amount(0)
{
    // can we do it?
    mustBePlayed(the_planet);

    // reserve everything for us
    reserved_sup = -transfer.changeCargo(el_Supplies,
                                         -transfer.getCargo(el_Supplies),
                                         true);
}

/** Destructor. */
GPlanetSellSuppliesAction::~GPlanetSellSuppliesAction()
{
    if (!isFinished()) {
        doCancel();
    }
}

/** Get maximum amount to sell. */
int32_t
GPlanetSellSuppliesAction::getMaximumAmount() const
{
    return reserved_sup;
}

/** Set amount to sell. Out-of-range values are automatically
    adjusted. */
void
GPlanetSellSuppliesAction::setAmount(int32_t amount)
{
    if (amount < 0)
        amount = 0;
    if (amount > reserved_sup)
        amount = reserved_sup;
    if (amount != this->amount) {
        this->amount = amount;
        sig_changed.raise();
    }
}

void
GPlanetSellSuppliesAction::commit()
{
    // commit:
    // - put back supplies
    // - add money
    transfer.changeCargo(el_Supplies, reserved_sup - amount, true);
    transfer.changeCargo(el_Money,    amount, true);
    transfer.doCommit();

    reserved_sup = amount = 0;
}

void
GPlanetSellSuppliesAction::cancel()
{
    // cancel: give back all supplies we have reserved, and don't change money.
    // To do that we just have to cancel the container.
    // (Since the container is a subtransaction, not a peer, we are allowed to cancel it.)
    transfer.doCancel();
    amount = reserved_sup = 0;
}
