/**
  *  \file int/export/expsep.cc
  *  \brief Export to Separated-Values file
  *
  *  This implements comma-separated values and derivatives.
  *
  *  Modelled partially after PCC 1.x export.pas::CTextExporter.
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/export/expsep.h"
#include "int/value.h"
#include "io/textfile.h"

IntSeparatedTextExporter::IntSeparatedTextExporter(TextFile& tf, char sep)
    : tf(tf), sep(sep), firstField(true)
{ }

void
IntSeparatedTextExporter::startTable(const IntExportFieldList& fields, const TIntTypeHint /*types*/[])
{
    /* Write a line with field names. Always quote field names.
       Otherwise, if the first field is "ID" (probably a common
       case), Excel would mistake the files as SYLK, not CSV. */
    for (IntExportFieldList::index_t i = 0; i < fields.size(); ++i) {
        if (i != 0) {
            tf.write(string_t(1, sep));
        }
        tf.write("\"");
        tf.write(fields.getFieldName(i));
        tf.write("\"");
    }
    tf.writeLine();
}

void
IntSeparatedTextExporter::startRecord()
{
    firstField = true;
}

void
IntSeparatedTextExporter::addField(IntValue* value, const string_t& /*name*/, TIntTypeHint /*type*/)
{
    // Separator
    if (!firstField) {
        tf.write(string_t(1, sep));
    }
    firstField = false;

    // Value. Remove newlines.
    string_t s = IntValue::toStringOf(value, false);
    string_t::size_type n = s.find('\n');
    if (n != s.npos) {
        s.erase(n);
        s.append("...");
    }

    // Quote by doubling quotes; this seems to be the standard way to quote
    // in CSV. PCC1.x quoted by using backslashes.
    if (s.find('"') != s.npos || s.find(sep) != s.npos) {
        for (string_t::size_type i = s.find('"'); i != s.npos; i = s.find('"', i+2)) {
            s.insert(i, 1, '"');
        }
        s.insert(0, "\"");
        s.append("\"");
    }
    tf.write(s);
}

void
IntSeparatedTextExporter::endRecord()
{
    tf.writeLine();
}

void
IntSeparatedTextExporter::endTable()
{ }
