/**
  *  \file int/ternary.cc
  *  \brief Interpreter: Ternary operations
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/ternary.h"
#include "int/keymap.h"
#include "cpluslib/misc.h"
#include "int/interror.h"
#include "util/atom.h"

/********************************* Naming ********************************/

static const char*const ternary_names[] = {
    "keyadd",
};

/** Get name for a ternary operation. This is used for disassembling.
    \param op Operation, IntTernaryOperation */
const char*
getTernaryName(uint8_t op)
{
    if (op < countof(ternary_names))
        return ternary_names[op];
    else
        return "?";
}

/******************************* Execution *******************************/

static IntValue*
FKeyAdd(IntValue* a, IntValue* b, IntValue* c)
{
    /* Check null operation */
    if (a == 0 || b == 0 || c == 0)
        return 0;

    /* a must be a keymap */
    IntKeymapValue* keymap = dynamic_cast<IntKeymapValue*>(a);
    if (!keymap)
        throw IntError::typeError(IntError::tyKeymap);

    /* b must be a string (the key) */
    IntStringValue* keysym = dynamic_cast<IntStringValue*>(b);
    if (!keysym)
        throw IntError::typeError(IntError::tyString);
    uint32_t keyval = parseKey(keysym->getValue());
    if (!keyval)
        throw IntError("Invalid key name");

    /* c must be a string or integer (the command) */
    uint32_t command;
    if (IntStringValue* sv = dynamic_cast<IntStringValue*>(c))
        command = atom(sv->getValue());
    else if (IntIntValue* iv = dynamic_cast<IntIntValue*>(c))
        command = iv->getValue();
    else
        throw IntError::typeError(IntError::tyString);

    /* Do it */
    keymap->getKeymap()->addKey(keyval, command, 0);

    return keymap->clone();
}

static IntValue* (*const ternary_ops[])(IntValue*,IntValue*,IntValue*) = {
    FKeyAdd,
};

/** Execute ternary operation.
    \param op Operation, IntTernaryOperation
    \param a,b,c User-supplied arguments taken from value stack
    \return New value to push on value stack */
IntValue*
executeTernaryOperation(uint8_t op, IntValue* a, IntValue* b, IntValue* c)
{
    if (op < countof(ternary_ops))
        return ternary_ops[op](a, b, c);
    else
        throw IntError::internalError("invalid ternary operation");
}
