/**
  *  \file ui/widgets/cardgroup.cc
  *  \brief UICardGroup widget
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widgets/cardgroup.h"
#include "gfx/gfxcontext.h"
#include "ui/skin.h"

/** Constructor. Build an empty card group. */
UICardGroup::UICardGroup(int id)
    : UIBaseWidget(id)
{
    sig_focus_changed.add(this, &UICardGroup::onFocusChange);
}

UICardGroup::~UICardGroup()
{ }

void
UICardGroup::onFocusChange(int)
{
    drawWidget();
}

void
UICardGroup::drawEverything(GfxCanvas& can)
{
    drawContent(can);
    if (UIBaseWidget* w = getFocusedChild())
        w->drawEverything(can);
}

void
UICardGroup::drawContent(GfxCanvas& canvas)
{
    drawBackground(GfxContext().useCanvas(canvas).useColorScheme(getSkin()), getExtent());
    canvas.update(getExtent());
}

void
UICardGroup::drawBelow(UIBaseWidget& /*child*/, GfxCanvas& canvas)
{
    /* We always draw everything even if the child is invisible, in case the
       child is trying to render the background into backing store */
    drawContent(canvas);
}

GfxCanvas&
UICardGroup::getCanvasFor(UIBaseWidget& child, ObjectHolder& h)
{
    if (&child == getFocusedChild())
        return getParent()->getCanvasFor(*this, h);
    else
        return h.add(new GfxNullCanvas());
}

bool
UICardGroup::handleEvent(const UIEvent& e, bool second_pass)
{
    if (UIBaseWidget* w = getFocusedChild())
        return w->handleEvent(e, second_pass);
    else
        return false;
}

void
UICardGroup::onAddRemoveChild(UIBaseWidget& /*the_widget*/, bool /*added*/)
{
    drawWidget();
}

void
UICardGroup::onChildResize(UIBaseWidget& the_widget, GfxRect /*oldsize*/)
{
    // our children should not change their size. Try the best we can.
    if (&the_widget == getFocusedChild())
        drawWidget();
    
}

void
UICardGroup::add(UIBaseWidget& w)
{
    w.setExtent(getExtent());
    addChildWidget(w, getLastChild());
}

void
UICardGroup::onResize(GfxRect /*oldsize*/, bool by_layout)
{
    for (UIBaseWidget* child = getFirstChild(); child != 0; child = child->getNextSibling()) {
        child->setExtent(getExtent(), true);
    }
    if (!by_layout)
        drawWidget();
}

void
UICardGroup::getLayoutInfo(LayoutInfo& info)
{
    LayoutInfo ch;

    info.flags = li_Flexible;
    info.min_size = info.pref_size = GfxPoint(1, 1);
    for (UIBaseWidget* child = getFirstChild(); child != 0; child = child->getNextSibling()) {
        child->getLayoutInfo(ch);
        info.flags &= ch.flags;
        if (ch.min_size.x > info.min_size.x)
            info.min_size.x = ch.min_size.x;
        if (ch.min_size.y > info.min_size.y)
            info.min_size.y = ch.min_size.y;
        if (ch.pref_size.x > info.pref_size.x)
            info.pref_size.x = ch.pref_size.x;
        if (ch.pref_size.y > info.pref_size.y)
            info.pref_size.y = ch.pref_size.y;
    }
}

